import {
  Button,
  Checkbox,
  Flex,
  Form,
  Modal,
  Result,
  Space,
  Typography,
  notification,
} from "antd";
import {
  Constants,
  EncodeStringToBase64,
  myFetch,
  setUserSessionToLocalStorage,
  showInputsInvalidNotification,
  showUnkownErrorNotification,
} from "../../utils";
import { useEffect, useRef, useState } from "react";
import {
  MyCompanyNameFormInput,
  MyEmailFormInput,
  MyPasswordFormInput,
  MyUsernameFormInput,
} from "../../Components/MyFormInputs";
import { Trans, useTranslation } from "react-i18next";
import MyAppLogo from "../../Components/MyAppLogo";
import { useNavigate } from "react-router-dom";
import ReCAPTCHA from "react-google-recaptcha";

export const AuthenticationMethod = {
  LOGIN: 1,
  SIGNUP: 2,
};

export default function Authentication({ method }) {
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  return (
    <>
      {notificationContextHolder}

      <Modal
        open={true}
        mask={false}
        closable={false}
        centered
        keyboard={false}
        footer={null}
      >
        <div
          style={{
            display: "flex",
            justifyContent: "center",
          }}
        >
          <MyAppLogo height={80} />
        </div>

        {method === AuthenticationMethod.LOGIN ? (
          <Login notificationApi={notificationApi} />
        ) : (
          <SignUp notificationApi={notificationApi} />
        )}
      </Modal>
    </>
  );
}

function RememberMeCheckbox() {
  const { t } = useTranslation();

  return (
    <Form.Item name="rememberMe" valuePropName="checked">
      <Checkbox>{t("authentication.rememberMe")}</Checkbox>
    </Form.Item>
  );
}

function PrivacyPolicyCheckbox() {
  const { t } = useTranslation();

  const link = (
    <a href={Constants.PRIVACY_POLICY_URL} target="_blank" rel="noreferrer" />
  );

  return (
    <Form.Item
      name="privacyPolicy"
      valuePropName="checked"
      rules={[
        {
          required: true,
          message: t("common.inputRules.privacyPolicyRequired"),
        },
      ]}
    >
      <Checkbox>
        <Trans
          i18nKey="authentication.privacyPolicy"
          values={{ dataPrivacy: t("authentication.privacyPolicyLink") }}
          components={{ dataLink: link }}
        />
      </Checkbox>
    </Form.Item>
  );
}

const LoginStep = {
  ACCOUNT_NAME: 1,
  PASSWORD: 2,
  PENDING_DELETION: 3,
  INIT_LOGIN: 4,
  BANNED: 5,
  _BACK_TO_PASSWORD: 6, // needed for going back from pending deletion to password step as the useEffect for the account name is triggered and would set the step to account name
  PENDING_EMAIL_VERIFICATION: 7,
};

// First step: account name -> get state of account by backend
// Second step: if account is active -> password -> login otherwise show the state of the account (e. g. deleted, banned)
function Login({ notificationApi }) {
  const { t } = useTranslation();
  const navigate = useNavigate();

  const [step, setStep] = useState(LoginStep.ACCOUNT_NAME);
  const [isRequesting, setIsRequesting] = useState(false);
  const recaptchaRef = useRef(null);
  const recaptchaValueRef = useRef(null);

  const [form] = Form.useForm();

  const email = Form.useWatch("email", form);

  const showErrorNotification = (errStatus) => {
    if (errStatus === 400) {
      notificationApi["error"]({
        message: t("authentication.login.request.400.title"),
        description: t("authentication.login.request.400.description"),
      });
    }
  };

  useEffect(() => {
    if (!email) return;

    // triggered if step was set to pending deletion and the user wants to reactivate the account
    if (step === LoginStep._BACK_TO_PASSWORD) {
      setStep(LoginStep.PASSWORD);
      return;
    }

    // reset step if account name changed for handling the state of the account
    if (step === LoginStep.PASSWORD || step === LoginStep.INIT_LOGIN) {
      setStep(LoginStep.ACCOUNT_NAME);
    }
  }, [email]);

  if (step === LoginStep.PENDING_DELETION) {
    return (
      <Result
        status="warning"
        title={t("authentication.login.stateAccountPendingDeletion.title")}
        subTitle={t(
          "authentication.login.stateAccountPendingDeletion.description"
        )}
        extra={[
          <Button key={0} onClick={() => setStep(LoginStep.ACCOUNT_NAME)}>
            {t("common.button.cancel")}
          </Button>,
          <Button
            key={1}
            type="primary"
            onClick={() => setStep(LoginStep._BACK_TO_PASSWORD)}
          >
            {t("authentication.login.stateAccountPendingDeletion.button")}
          </Button>,
        ]}
      />
    );
  }

  if (step === LoginStep.BANNED) {
    return (
      <Result
        status="error"
        title={t("authentication.login.stateAccountBanned.title")}
        subTitle={
          <Space direction="vertical">
            <Typography.Text type="secondary">
              {t("authentication.login.stateAccountBanned.infoSupport")}
            </Typography.Text>

            <Button type="link" href={`mailto:${Constants.SUPPORT_EMAIL}`}>
              {Constants.SUPPORT_EMAIL}
            </Button>
          </Space>
        }
        extra={[
          <Button
            key={0}
            onClick={() => {
              setStep(LoginStep.ACCOUNT_NAME);
            }}
          >
            {t("authentication.login.stateAccountBanned.backButton")}
          </Button>,
        ]}
      />
    );
  }

  if (step === LoginStep.PENDING_EMAIL_VERIFICATION) {
    return <PendingEmailVerification />;
  }

  return (
    <Form
      form={form}
      layout="vertical"
      requiredMark={false}
      initialValues={{
        rememberMe: false,
      }}
    >
      {step === LoginStep.INIT_LOGIN && (
        <Typography.Paragraph type="secondary">
          {t("authentication.login.stateInitLogin.info")}
        </Typography.Paragraph>
      )}

      <MyEmailFormInput hasFeedback={false} disableEmailCheck />

      <div
        style={{
          display:
            step === LoginStep.PASSWORD || step === LoginStep.INIT_LOGIN
              ? "block"
              : "none",
        }}
      >
        <MyPasswordFormInput />

        <RememberMeCheckbox />

        <Form.Item
          name="recaptcha"
          rules={[
            {
              required: true,
              message: t("common.inputRules.recaptchaRequired"),
            },
          ]}
        >
          <ReCAPTCHA
            ref={recaptchaRef}
            sitekey={process.env.REACT_APP_RECAPTCHA_SITE_KEY}
            onChange={(value) => (recaptchaValueRef.current = value)}
          />
        </Form.Item>

        <PrivacyPolicyCheckbox />
      </div>

      <Button
        type="primary"
        size="large"
        block
        loading={isRequesting}
        htmlType="submit"
        onClick={() => {
          let validateFields = [];

          if (step === LoginStep.ACCOUNT_NAME) {
            validateFields = ["email"];
          } else {
            validateFields = [
              "email",
              "password",
              "rememberMe",
              "recaptcha",
              "privacyPolicy",
            ];
          }

          form
            .validateFields(validateFields)
            .then((values) => {
              setIsRequesting(true);

              let body = {
                email: values.email.toLocaleLowerCase(),
              };

              if (
                step === LoginStep.PASSWORD ||
                step === LoginStep.INIT_LOGIN
              ) {
                body.password = EncodeStringToBase64(values.password);
                body.rememberMe = values.rememberMe;
                body.recaptcha = recaptchaValueRef.current;
              }

              myFetch({
                url: `/user/auth/login`,
                method: "POST",
                body: body,
                notificationApi: notificationApi,
                t: t,
              })
                .then((data) => {
                  setIsRequesting(false);

                  if (
                    step === LoginStep.PASSWORD ||
                    step === LoginStep.INIT_LOGIN
                  ) {
                    setUserSessionToLocalStorage(data.XAuthorization);
                    window.location.href = "/";
                    return;
                  }

                  if (data.state === undefined) return;

                  switch (data.state) {
                    case Constants.ACCOUNT_STATE.ACTIVE:
                      setStep(LoginStep.PASSWORD);
                      break;
                    case Constants.ACCOUNT_STATE.PENDING_DELETION:
                      setStep(LoginStep.PENDING_DELETION);
                      break;
                    case Constants.ACCOUNT_STATE.BANNED:
                      setStep(LoginStep.BANNED);
                      break;
                    case Constants.ACCOUNT_STATE.INIT_LOGGING:
                      setStep(LoginStep.INIT_LOGIN);
                      break;
                    case Constants.ACCOUNT_STATE.PENDING_EMAIL_VERIFICATION:
                      setStep(LoginStep.PENDING_EMAIL_VERIFICATION);
                      break;
                    default:
                      showUnkownErrorNotification(notificationApi, t);
                      break;
                  }
                })
                .catch((errStatus) => {
                  showErrorNotification(errStatus);
                  setIsRequesting(false);
                  recaptchaRef.current.reset();
                });
            })
            .catch(() => showInputsInvalidNotification(notificationApi, t));
        }}
      >
        {t("authentication.login.button")}
      </Button>

      <Flex justify="center" style={{ paddingTop: 12 }}>
        <Typography.Text>
          {t("authentication.login.dontHaveAccount")}{" "}
          <Button
            type="link"
            style={{ padding: 0 }}
            onClick={() =>
              navigate(Constants.ROUTE_PATHS.AUTHENTICATION.SIGN_UP)
            }
          >
            {t("authentication.signUpLink")}
          </Button>
        </Typography.Text>
      </Flex>
    </Form>
  );
}

const SignUpStep = {
  SIGN_UP: 1,
  PENDING_EMAIL_VERIFICATION: 2,
};

function SignUp({ notificationApi }) {
  const { t, i18n } = useTranslation();
  const navigate = useNavigate();

  const [step, setStep] = useState(SignUpStep.SIGN_UP);
  const [isRequesting, setIsRequesting] = useState(false);
  const recaptchaValueRef = useRef(null);

  const [form] = Form.useForm();

  const showErrorNotification = (errStatus) => {
    if (errStatus === 400) {
      notificationApi["error"]({
        message: t("authentication.signUp.request.400.title"),
        description: t("authentication.signUp.request.400.description"),
      });
    }
  };

  const handleSubmit = () => {
    form
      .validateFields()
      .then((values) => {
        setIsRequesting(true);

        myFetch({
          url: `/user/auth/signup`,
          method: "POST",
          body: {
            companyName: values.companyName,
            email: values.email.toLocaleLowerCase(),
            password: EncodeStringToBase64(values.password),
            username: values.username,
            language: i18n.language,
            rememberMe: values.rememberMe,
            recaptcha: recaptchaValueRef.current,
          },
          notificationApi: notificationApi,
          t: t,
        })
          .then(() => setStep(SignUpStep.PENDING_EMAIL_VERIFICATION))
          .catch((errStatus) => {
            showErrorNotification(errStatus);
            setIsRequesting(false);
          });
      })
      .catch(() => {
        showInputsInvalidNotification(notificationApi, t);
        setIsRequesting(false);
      });
  };

  if (step === SignUpStep.PENDING_EMAIL_VERIFICATION) {
    return <PendingEmailVerification />;
  }

  return (
    <Form
      form={form}
      layout="vertical"
      requiredMark={false}
      initialValues={{
        rememberMe: false,
      }}
    >
      <MyCompanyNameFormInput />

      <MyUsernameFormInput
        propsFormItem={{ extra: t("common.usernameInfo") }}
      />

      <MyEmailFormInput hasFeedback={true} />

      <MyPasswordFormInput />

      <RememberMeCheckbox />

      <Form.Item
        name="recaptcha"
        rules={[
          {
            required: true,
            message: t("common.inputRules.recaptchaRequired"),
          },
        ]}
      >
        <ReCAPTCHA
          sitekey={process.env.REACT_APP_RECAPTCHA_SITE_KEY}
          onChange={(value) => (recaptchaValueRef.current = value)}
        />
      </Form.Item>

      <PrivacyPolicyCheckbox />

      <Button
        type="primary"
        size="large"
        block
        onClick={handleSubmit}
        loading={isRequesting}
      >
        {t("authentication.signUp.button")}
      </Button>

      <Flex justify="center" style={{ paddingTop: 12 }}>
        <Typography.Text>
          {t("authentication.signUp.alreadyHaveAccount")}{" "}
          <Button
            type="link"
            style={{ padding: 0 }}
            onClick={() => navigate(Constants.ROUTE_PATHS.AUTHENTICATION.LOGIN)}
          >
            {t("authentication.loginLink")}
          </Button>
        </Typography.Text>
      </Flex>
    </Form>
  );
}

function PendingEmailVerification() {
  const { t } = useTranslation();

  return (
    <Result
      status="warning"
      title={t("authentication.signUp.pendingEmailVerification.title")}
      subTitle={t("authentication.signUp.pendingEmailVerification.description")}
    />
  );
}
